// An example of a DDA-like algorithm to step through all grid cells that
// touch a ray and get the cell entry points.
//
// For public domain, 6.1.2014, Jere "XMunkki" Sanisalo, http://www.xmunkki.org/
//
// Use Processingjs.org to test.

void setup() {
    frameRate(30);
    
    size(700, 375);
}

// Converts a coordinate component to screen coordinates
float c2s(float c)
{
    return c * 35.0;
}

int MODE_BG = 0;
int MODE_HIT_PT = 1;

void traceLine(float x1, float y1, float x2, float y2, int mode) {
    int x = floor(x1);
    int y = floor(y1);
    int dest_x = floor(x2);
    int dest_y = floor(y2);
    
    // Calculate the direction
    float line_len = mag(x2 - x1, y2 - y1);
    float xd = (x2 - x1) / line_len;
    float yd = (y2 - y1) / line_len;
    
    // Calculate the delta in x/y for one grid unit
    float delta_x = sqrt(1.0 + (yd*yd)/(xd*xd));
    float delta_y = sqrt(1.0 + (xd*xd)/(yd*yd));
    
    // Calculate the step size
    int step_x, step_y;
    float dist_x, dist_y;
    
    if (xd >= 0) {
        step_x = 1;
        dist_x = (x + 1.0 - x1) * delta_x;
    } else {
        step_x = -1;
        dist_x = (x1 - x) * delta_x;
    }
    
    if (yd >= 0) {
        step_y = 1;
        dist_y = (y + 1.0 - y1) * delta_y;
    } else {
        step_y = -1;
        dist_y = (y1 - y) * delta_y;
    }
    
    // Draw the line
    float moved_dist = 0;
    
    for (int steps = 0; steps < 1000; ++steps)
    {
        // Fill the current position
        if (mode == MODE_BG)
        {
            noStroke();
            fill(50, 50, 50);
            rect(c2s(x), c2s(y), c2s(1), c2s(1));
        }
        
        // Done?
        if (x == dest_x && y == dest_y)
            break;
        
        // Next step
        if (dist_x < dist_y)
        {
            moved_dist = dist_x;
            dist_x += delta_x;
            
            //// Or: (incremental moved_dist)
            // moved_dist += dist_x;
            // dist_y -= dist_x;
            // dist_x = delta_x;
            
            x += step_x;
        }
        else
        {
            moved_dist = dist_y;
            dist_y += delta_y;

            //// Or: (incremental moved_dist)
            // moved_dist += dist_y;
            // dist_x -= dist_y;
            // dist_y = delta_y;
            
            y += step_y;
        }
        
        // Draw the hit points
        if (mode == MODE_HIT_PT)
        {
            float hit_x = x1 + xd * moved_dist;
            float hit_y = y1 + yd * moved_dist;
            float s = c2s(0.1);
            
            stroke(0);
            fill(50, 255, 50);
            rect(c2s(hit_x) - s, c2s(hit_y) - s, s * 2.0, s * 2.0);
        }
    }
}

void draw() {
    float time = frameCount / 30.0;

    // Create the line
    float ang = 15.0 + time * 0.1;
    float x1 = 10 + cos(ang * 0.93) * 4;
    float y1 = 6 + sin(ang) * 4;
    ang *= -2.812345;
    float x2 = 10 + cos(ang) * 4;
    float y2 = 6 + sin(ang * 0.95) * 4;

    // Clear the screen
    background(0);

    // Trace the line (bg)
    traceLine(x1, y1, x2, y2, MODE_BG);
    
    // Draw the grid
    stroke(125);
    for (int i = 0; i < 100; ++i)
    {
        float xc = c2s(i);
        float yc = c2s(i);
        float mc = c2s(100);
        line(xc, 0, xc, mc);
        line(0, yc, mc, yc);
    }
  
    // Draw the line
    stroke(255);
    line(c2s(x1), c2s(y1), c2s(x2), c2s(y2));

    // Trace the line (hit points)
    traceLine(x1, y1, x2, y2, MODE_HIT_PT);
}